==> rest-sdk-connector-model-0.3.0-HF1-sources.jar: com/mulesoft/connectivity/rest/sdk/internal/connectormodel/builder/ConnectorModelBuilder.java <==
/*
 * (c) 2003-2021 MuleSoft, Inc. This software is protected under international copyright
 * law. All use of this software is subject to MuleSoft's Master Subscription Agreement
 * (or other master license agreement) separately entered into in writing between you and
 * MuleSoft. If such an agreement is not in place, you may not use the software.
 */
package com.mulesoft.connectivity.rest.sdk.internal.connectormodel.builder;

import static com.mulesoft.connectivity.rest.sdk.internal.connectormodel.Protocol.HTTP;
import static com.mulesoft.connectivity.rest.sdk.internal.connectormodel.Protocol.HTTPS;
import static com.mulesoft.connectivity.rest.sdk.internal.connectormodel.QueryParamArrayFormat.MULTIMAP;
import static com.mulesoft.connectivity.rest.sdk.internal.connectormodel.security.ConnectorSecurityScheme.SecuritySchemeType.UNSECURED;
import static com.mulesoft.connectivity.rest.sdk.internal.webapi.util.XmlUtils.getXmlName;
import static com.mulesoft.connectivity.rest.sdk.internal.webapi.util.XmlUtils.removeMavenArtifactUnwantedCharacters;
import static com.mulesoft.connectivity.rest.sdk.internal.webapi.util.XmlUtils.removeMavenGroupUnwantedCharacters;
import static com.mulesoft.connectivity.rest.sdk.internal.webapi.util.XmlUtils.removeMavenVersionUnwantedCharacters;
import static java.lang.String.format;
import static java.util.Arrays.asList;
import static java.util.Objects.requireNonNull;
import static java.util.stream.Collectors.toList;
import static org.apache.commons.lang3.ObjectUtils.defaultIfNull;
import static org.apache.commons.lang3.StringUtils.isNotBlank;
import org.mule.weave.v2.parser.phase.CompilationException;

import com.mulesoft.connectivity.rest.sdk.internal.connectormodel.ConnectorCategory;
import com.mulesoft.connectivity.rest.sdk.internal.connectormodel.ConnectorModel;
import com.mulesoft.connectivity.rest.sdk.internal.connectormodel.ConnectorOperation;
import com.mulesoft.connectivity.rest.sdk.internal.connectormodel.ConnectorPackage;
import com.mulesoft.connectivity.rest.sdk.internal.connectormodel.HTTPMethod;
import com.mulesoft.connectivity.rest.sdk.internal.connectormodel.Protocol;
import com.mulesoft.connectivity.rest.sdk.internal.connectormodel.QueryParamArrayFormat;
import com.mulesoft.connectivity.rest.sdk.internal.connectormodel.TypeSchemaPool;
import com.mulesoft.connectivity.rest.sdk.internal.connectormodel.dw.OperationDisplayNameExpressionHandler;
import com.mulesoft.connectivity.rest.sdk.internal.connectormodel.dw.OperationIdentifierExpressionHandler;
import com.mulesoft.connectivity.rest.sdk.internal.connectormodel.pagination.Pagination;
import com.mulesoft.connectivity.rest.sdk.internal.connectormodel.security.ConnectorSecurityScheme;
import com.mulesoft.connectivity.rest.sdk.internal.connectormodel.security.TestConnectionConfig;
import com.mulesoft.connectivity.rest.sdk.internal.webapi.exception.ModelGenerationException;

import java.util.ArrayList;
import java.util.List;

import javax.ws.rs.core.MediaType;

public class ConnectorModelBuilder {

  private final static Protocol[] DEFAULT_PROTOCOLS = {HTTP, HTTPS};

  private String connectorName;
  private String connectorDescription;
  private String mavenGroup;
  private String mavenArtifactId;
  private String mavenVersion;
  private String baseJavaPackage;
  private ConnectorCategory connectorCategory;
  private List<Protocol> supportedProtocols;
  private String extensionXml;
  private QueryParamArrayFormat queryParamArrayFormat;
  private final List<PaginationBuilder> paginationBuilders = new ArrayList<>();
  private final List<OperationBuilder> operationBuilders = new ArrayList<>();
  private final List<TriggerBuilder> triggerBuilders = new ArrayList<>();
  private final List<SecuritySchemeBuilder> securitySchemeBuilders = new ArrayList<>();
  private final BaseUriBuilder baseUriBuilder = new BaseUriBuilder();
  private Boolean skipOutputTypeValidation;
  private String operationIdentifierExpression;
  private String operationDisplayNameExpression;
  private MediaType defaultInputMediaType;
  private MediaType defaultOutputMediaType;
  private TestConnectionConfig testConnectionConfig;

  public ConnectorModelBuilder name(String connectorName) {
    this.connectorName = defaultIfNull(connectorName, this.connectorName);
    return this;
  }

  public ConnectorModelBuilder description(String connectorDescription) {
    this.connectorDescription = defaultIfNull(connectorDescription, this.connectorDescription);;
    return this;
  }

  public ConnectorModelBuilder mavenGroup(String mavenGroup) {
    this.mavenGroup = defaultIfNull(mavenGroup, this.mavenGroup);;
    return this;
  }

  public ConnectorModelBuilder mavenArtifactId(String mavenArtifactId) {
    this.mavenArtifactId = defaultIfNull(mavenArtifactId, this.mavenArtifactId);;
    return this;
  }

  public ConnectorModelBuilder mavenVersion(String mavenVersion) {
    this.mavenVersion = defaultIfNull(mavenVersion, this.mavenVersion);;
    return this;
  }

  public ConnectorModelBuilder baseJavaPackage(String baseJavaPackage) {
    this.baseJavaPackage = defaultIfNull(baseJavaPackage, this.baseJavaPackage);;
    return this;
  }

  public ConnectorModelBuilder category(ConnectorCategory connectorCategory) {
    this.connectorCategory = defaultIfNull(connectorCategory, this.connectorCategory);;
    return this;
  }

  public ConnectorModelBuilder supportedProtocols(List<Protocol> protocols) {
    this.supportedProtocols = defaultIfNull(protocols, this.supportedProtocols);;
    return this;
  }

  public ConnectorModelBuilder queryParamArrayFormat(QueryParamArrayFormat queryParamArrayFormat) {
    this.queryParamArrayFormat = defaultIfNull(queryParamArrayFormat, this.queryParamArrayFormat);;
    return this;
  }

  public ConnectorModelBuilder extensionXml(String extensionXml) {
    this.extensionXml = defaultIfNull(extensionXml, this.extensionXml);;
    return this;
  }

  public BaseUriBuilder getBaseUriBuilder() {
    return this.baseUriBuilder;
  }

  public Boolean getSkipOutputTypeValidation() {
    return this.skipOutputTypeValidation;
  }

  public ConnectorModelBuilder skipOutputTypeValidation(Boolean skipOutputTypeValidation) {
    this.skipOutputTypeValidation = defaultIfNull(skipOutputTypeValidation, this.skipOutputTypeValidation);;
    return this;
  }

  public ConnectorModelBuilder operationDisplayNameExpression(String operationDisplayNameExpression) {
    this.operationDisplayNameExpression = defaultIfNull(operationDisplayNameExpression, this.operationDisplayNameExpression);;
    return this;
  }

  public ConnectorModelBuilder operationIdentifierExpression(String operationIdentifierExpression) {
    if (this.operationIdentifierExpression != null) {
      throw new IllegalArgumentException("operationIdentifierExpression cannot be overridden.");
    }
    this.operationIdentifierExpression = operationIdentifierExpression;
    return this;
  }

  public MediaType getDefaultInputMediaType() {
    return defaultInputMediaType;
  }

  public ConnectorModelBuilder defaultInputMediaType(MediaType defaultInputMediaType) {
    this.defaultInputMediaType = defaultInputMediaType;
    return this;
  }

  public MediaType getDefaultOutputMediaType() {
    return defaultOutputMediaType;
  }

  public ConnectorModelBuilder defaultOutputMediaType(MediaType defaultOutputMediaType) {
    this.defaultOutputMediaType = defaultOutputMediaType;
    return this;
  }

  public ConnectorModelBuilder testConnection(TestConnectionConfig testConnectionConfig) {
    this.testConnectionConfig = testConnectionConfig;
    return this;
  }

  public List<OperationBuilder> getOperationBuilders() {
    return operationBuilders;
  }

  public List<OperationBuilder> getOperationBuildersByPath(String path) {
    return operationBuilders.stream()
        .filter(x -> x.getPath().equalsIgnoreCase(path))
        .collect(toList());
  }

  public OperationBuilder getOrCreateOperationBuilder(String apiOperationId, String apiPath, String apiMethod,
                                                      String apiSummary) {
    requireNonNull(apiMethod, "API Method is required");
    requireNonNull(apiPath, "API Path is required");

    OperationBuilder operationBuilder =
        operationBuilders.stream()
            .filter(x -> x.getPath().equalsIgnoreCase(apiPath) && x.getMethod().equalsIgnoreCase(apiMethod))
            .findFirst().orElse(null);

    if (operationBuilder == null) {
      operationBuilder = new OperationBuilder(apiOperationId, apiPath, apiMethod, apiSummary);
      operationBuilders.add(operationBuilder);
    }

    return operationBuilder;
  }

  public PaginationBuilder getOrCreatePaginationBuilder(String paginationName) {
    requireNonNull(paginationName, "Pagination Name is required");

    PaginationBuilder paginationBuilder =
        paginationBuilders.stream()
            .filter(x -> x.getName().equals(paginationName))
            .findFirst().orElse(null);

    if (paginationBuilder == null) {
      paginationBuilder = new PaginationBuilder(paginationName);
      paginationBuilders.add(paginationBuilder);
    }

    return paginationBuilder;
  }

  public SecuritySchemeBuilder getOrCreateSecuritySchemeBuilder(String name,
                                                                ConnectorSecurityScheme.SecuritySchemeType type) {
    requireNonNull(name, "Security scheme name is required");

    SecuritySchemeBuilder securitySchemeBuilder =
        securitySchemeBuilders.stream()
            .filter(x -> x.getName().equals(name))
            .findFirst().orElse(null);

    if (securitySchemeBuilder == null) {
      requireNonNull(type, format("Security Scheme named %s does not exist and a type was not specified. This is a bug.", name));
      securitySchemeBuilder = new SecuritySchemeBuilder(name, type);
      securitySchemeBuilders.add(securitySchemeBuilder);
    }

    if (type != null && !securitySchemeBuilder.getType().equals(type)) {
      throw new IllegalStateException("Security scheme type does not match. This is a bug.");
    }

    return securitySchemeBuilder;
  }

  public List<TriggerBuilder> getTriggerBuilders() {
    return triggerBuilders;
  }

  public TriggerBuilder getOrCreateTriggerBuilder(String name) {
    requireNonNull(name, "Trigger name is required");

    TriggerBuilder triggerBuilder =
        triggerBuilders.stream()
            .filter(x -> x.getName().equals(name))
            .findFirst().orElse(null);

    if (triggerBuilder == null) {
      triggerBuilder = new TriggerBuilder(name);
      triggerBuilders.add(triggerBuilder);
    }

    return triggerBuilder;
  }

  public void setAllOperationsIgnored(Boolean ignoreOperations) {
    operationBuilders.forEach(x -> x.ignored(ignoreOperations));
    triggerBuilders.forEach(x -> x.ignored(ignoreOperations));
  }

  public ConnectorModel build() throws ModelGenerationException {
    requireNonNull(connectorName, "Connector name is required");

    TypeSchemaPool typeSchemaPool = new TypeSchemaPool();

    List<Pagination> paginations = paginationBuilders.stream().map(PaginationBuilder::build).collect(toList());

    OperationDisplayNameExpressionHandler operationDisplayNameExpressionHandler = buildOperationDisplayNameHandler();
    OperationIdentifierExpressionHandler operationIdentifierExpressionHandler = buildOperationIdentifierHandler();

    List<ConnectorSecurityScheme> securitySchemes = buildSecuritySchemes(typeSchemaPool);

    List<ConnectorOperation> operations = operationBuilders.stream()
        .map(x -> x.build(operationIdentifierExpressionHandler,
                          operationDisplayNameExpressionHandler,
                          typeSchemaPool,
                          paginations,
                          securitySchemes,
                          defaultInputMediaType,
                          defaultOutputMediaType,
                          buildQueryParamArrayFormat(),
                          skipOutputTypeValidation))
        .collect(toList());

    return new ConnectorModel(buildConnectorName(),
                              connectorDescription,
                              buildMavenGroup(),
                              buildMavenArtifactId(),
                              buildMavenVersion(),
                              buildBasePackage(),
                              connectorCategory,
                              baseUriBuilder.build(supportedProtocols),
                              getSupportedProtocols(),
                              operations.stream().filter(x -> !x.isIgnored()).collect(toList()),
                              paginations,
                              buildExtensionXml(),
                              triggerBuilders.stream().filter(x -> !x.isIgnored())
                                  .map(x -> x.build(operations, typeSchemaPool)).collect(toList()));
  }

  private List<ConnectorSecurityScheme> buildSecuritySchemes(TypeSchemaPool typeSchemaPool) {
    securitySchemeBuilders.add(new SecuritySchemeBuilder(UNSECURED.toString(), UNSECURED));

    return securitySchemeBuilders.stream()
        .filter(x -> x.isIgnored() == null || !x.isIgnored())
        .map(x -> x.build(typeSchemaPool, testConnectionConfig))
        .collect(toList());
  }

  private String buildConnectorName() {
    String builtName = connectorName.trim();

    if (builtName.toLowerCase().endsWith("api")) {
      builtName = builtName.substring(0, builtName.lastIndexOf("api"));
    }
    if (builtName.toLowerCase().endsWith("connector")) {
      return builtName;
    }
    return builtName + " Connector";
  }

  private String buildMavenGroup() {
    return removeMavenGroupUnwantedCharacters(mavenGroup);
  }

  private String buildMavenArtifactId() {
    return removeMavenArtifactUnwantedCharacters(mavenArtifactId);
  }

  private String buildMavenVersion() {
    return removeMavenVersionUnwantedCharacters(mavenVersion);
  }

  private List<Protocol> getSupportedProtocols() {
    return (supportedProtocols != null && !supportedProtocols.isEmpty()) ? supportedProtocols : asList(DEFAULT_PROTOCOLS);
  }

  private String buildBasePackage() {
    return isNotBlank(baseJavaPackage) ? baseJavaPackage : ConnectorPackage.buildBasePackage(connectorName);
  }

  private String buildExtensionXml() {
    return isNotBlank(extensionXml) ? extensionXml : getXmlName(connectorName);
  }

  private QueryParamArrayFormat buildQueryParamArrayFormat() {
    return defaultIfNull(queryParamArrayFormat, MULTIMAP);
  }

  private OperationIdentifierExpressionHandler buildOperationIdentifierHandler() {
    try {
      return isNotBlank(operationIdentifierExpression)
          ? new OperationIdentifierExpressionHandler(operationIdentifierExpression)
          : new OperationIdentifierExpressionHandler();

    } catch (CompilationException e) {
      throw new IllegalArgumentException("The script for the 'operationIdentifier' failed to compile. This is a bug.", e);
    }
  }

  private OperationDisplayNameExpressionHandler buildOperationDisplayNameHandler() {
    try {
      return isNotBlank(operationDisplayNameExpression)
          ? new OperationDisplayNameExpressionHandler(operationDisplayNameExpression)
          : new OperationDisplayNameExpressionHandler();
    } catch (CompilationException e) {
      throw new IllegalArgumentException("The script for the 'operationDisplayName' failed to compile. This is a bug.", e);
    }
  }
}

==> rest-sdk-connector-model-0.4.1-HF1-sources.jar: com/mulesoft/connectivity/rest/sdk/internal/connectormodel/builder/ConnectorModelBuilder.java <==
/*
 * (c) 2003-2021 MuleSoft, Inc. This software is protected under international copyright
 * law. All use of this software is subject to MuleSoft's Master Subscription Agreement
 * (or other master license agreement) separately entered into in writing between you and
 * MuleSoft. If such an agreement is not in place, you may not use the software.
 */
package com.mulesoft.connectivity.rest.sdk.internal.connectormodel.builder;

import static com.mulesoft.connectivity.rest.sdk.internal.connectormodel.Protocol.HTTP;
import static com.mulesoft.connectivity.rest.sdk.internal.connectormodel.Protocol.HTTPS;
import static com.mulesoft.connectivity.rest.sdk.internal.connectormodel.QueryParamArrayFormat.MULTIMAP;
import static com.mulesoft.connectivity.rest.sdk.internal.connectormodel.security.ConnectorSecurityScheme.SecuritySchemeType.UNSECURED;
import static com.mulesoft.connectivity.rest.sdk.internal.webapi.util.XmlUtils.getXmlName;
import static com.mulesoft.connectivity.rest.sdk.internal.webapi.util.XmlUtils.removeMavenArtifactUnwantedCharacters;
import static com.mulesoft.connectivity.rest.sdk.internal.webapi.util.XmlUtils.removeMavenGroupUnwantedCharacters;
import static com.mulesoft.connectivity.rest.sdk.internal.webapi.util.XmlUtils.removeMavenVersionUnwantedCharacters;
import static java.lang.String.format;
import static java.util.Arrays.asList;
import static java.util.Objects.requireNonNull;
import static java.util.stream.Collectors.toList;
import static org.apache.commons.lang3.ObjectUtils.defaultIfNull;
import static org.apache.commons.lang3.StringUtils.isNotBlank;
import org.mule.weave.v2.parser.phase.CompilationException;

import com.mulesoft.connectivity.rest.sdk.internal.connectormodel.ConnectorCategory;
import com.mulesoft.connectivity.rest.sdk.internal.connectormodel.ConnectorModel;
import com.mulesoft.connectivity.rest.sdk.internal.connectormodel.ConnectorOperation;
import com.mulesoft.connectivity.rest.sdk.internal.connectormodel.ConnectorPackage;
import com.mulesoft.connectivity.rest.sdk.internal.connectormodel.Protocol;
import com.mulesoft.connectivity.rest.sdk.internal.connectormodel.QueryParamArrayFormat;
import com.mulesoft.connectivity.rest.sdk.internal.connectormodel.TypeSchemaPool;
import com.mulesoft.connectivity.rest.sdk.internal.connectormodel.dw.OperationDisplayNameExpressionHandler;
import com.mulesoft.connectivity.rest.sdk.internal.connectormodel.dw.OperationIdentifierExpressionHandler;
import com.mulesoft.connectivity.rest.sdk.internal.connectormodel.pagination.Pagination;
import com.mulesoft.connectivity.rest.sdk.internal.connectormodel.security.ConnectorSecurityScheme;
import com.mulesoft.connectivity.rest.sdk.internal.connectormodel.security.TestConnectionConfig;
import com.mulesoft.connectivity.rest.sdk.internal.webapi.exception.ModelGenerationException;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

import javax.ws.rs.core.MediaType;

public class ConnectorModelBuilder {

  private final static Protocol[] DEFAULT_PROTOCOLS = {HTTP, HTTPS};

  private String connectorName;
  private String connectorDescription;
  private String mavenGroup;
  private String mavenArtifactId;
  private String mavenVersion;
  private String baseJavaPackage;
  private ConnectorCategory connectorCategory;
  private List<Protocol> supportedProtocols;
  private String extensionXml;
  private QueryParamArrayFormat queryParamArrayFormat;
  private final List<PaginationBuilder> paginationBuilders = new ArrayList<>();
  private final List<OperationBuilder> operationBuilders = new ArrayList<>();
  private final List<TriggerBuilder> triggerBuilders = new ArrayList<>();
  private final List<SecuritySchemeBuilder> securitySchemeBuilders = new ArrayList<>();
  private final BaseUriBuilder baseUriBuilder = new BaseUriBuilder();
  private Boolean skipOutputTypeValidation;
  private String operationIdentifierExpression;
  private String operationDisplayNameExpression;
  private MediaType defaultInputMediaType;
  private MediaType defaultOutputMediaType;
  private TestConnectionConfig testConnectionConfig;

  public ConnectorModelBuilder name(String connectorName) {
    this.connectorName = defaultIfNull(connectorName, this.connectorName);
    return this;
  }

  public ConnectorModelBuilder description(String connectorDescription) {
    this.connectorDescription = defaultIfNull(connectorDescription, this.connectorDescription);;
    return this;
  }

  public ConnectorModelBuilder mavenGroup(String mavenGroup) {
    this.mavenGroup = defaultIfNull(mavenGroup, this.mavenGroup);;
    return this;
  }

  public ConnectorModelBuilder mavenArtifactId(String mavenArtifactId) {
    this.mavenArtifactId = defaultIfNull(mavenArtifactId, this.mavenArtifactId);;
    return this;
  }

  public ConnectorModelBuilder mavenVersion(String mavenVersion) {
    this.mavenVersion = defaultIfNull(mavenVersion, this.mavenVersion);;
    return this;
  }

  public ConnectorModelBuilder baseJavaPackage(String baseJavaPackage) {
    this.baseJavaPackage = defaultIfNull(baseJavaPackage, this.baseJavaPackage);;
    return this;
  }

  public ConnectorModelBuilder category(ConnectorCategory connectorCategory) {
    this.connectorCategory = defaultIfNull(connectorCategory, this.connectorCategory);;
    return this;
  }

  public ConnectorModelBuilder supportedProtocols(List<Protocol> protocols) {
    this.supportedProtocols = defaultIfNull(protocols, this.supportedProtocols);;
    return this;
  }

  public ConnectorModelBuilder queryParamArrayFormat(QueryParamArrayFormat queryParamArrayFormat) {
    this.queryParamArrayFormat = defaultIfNull(queryParamArrayFormat, this.queryParamArrayFormat);;
    return this;
  }

  public ConnectorModelBuilder extensionXml(String extensionXml) {
    this.extensionXml = defaultIfNull(extensionXml, this.extensionXml);;
    return this;
  }

  public BaseUriBuilder getBaseUriBuilder() {
    return this.baseUriBuilder;
  }

  public Boolean getSkipOutputTypeValidation() {
    return this.skipOutputTypeValidation;
  }

  public ConnectorModelBuilder skipOutputTypeValidation(Boolean skipOutputTypeValidation) {
    this.skipOutputTypeValidation = defaultIfNull(skipOutputTypeValidation, this.skipOutputTypeValidation);;
    return this;
  }

  public ConnectorModelBuilder operationDisplayNameExpression(String operationDisplayNameExpression) {
    this.operationDisplayNameExpression = defaultIfNull(operationDisplayNameExpression, this.operationDisplayNameExpression);;
    return this;
  }

  public ConnectorModelBuilder operationIdentifierExpression(String operationIdentifierExpression) {
    if (this.operationIdentifierExpression != null) {
      throw new IllegalArgumentException("operationIdentifierExpression cannot be overridden.");
    }
    this.operationIdentifierExpression = operationIdentifierExpression;
    return this;
  }

  public MediaType getDefaultInputMediaType() {
    return defaultInputMediaType;
  }

  public ConnectorModelBuilder defaultInputMediaType(MediaType defaultInputMediaType) {
    this.defaultInputMediaType = defaultInputMediaType;
    return this;
  }

  public MediaType getDefaultOutputMediaType() {
    return defaultOutputMediaType;
  }

  public ConnectorModelBuilder defaultOutputMediaType(MediaType defaultOutputMediaType) {
    this.defaultOutputMediaType = defaultOutputMediaType;
    return this;
  }

  public ConnectorModelBuilder testConnection(TestConnectionConfig testConnectionConfig) {
    this.testConnectionConfig = testConnectionConfig;
    return this;
  }

  public List<OperationBuilder> getOperationBuilders() {
    return operationBuilders;
  }

  public List<OperationBuilder> getOperationBuildersByPath(String path) {
    return operationBuilders.stream()
        .filter(x -> x.getPath().equalsIgnoreCase(path))
        .collect(toList());
  }

  public Optional<OperationBuilder> getOperationBuildersByOperationId(String operationId) {
    return operationBuilders.stream()
        .filter(x -> operationId(x).equalsIgnoreCase(operationId))
        .findFirst();
  }

  // TODO
  private String operationId(OperationBuilder operationBuilder) {
    return Optional.ofNullable(operationBuilder.getApiOperationId())
        .orElse(String.format("%s-%s", operationBuilder.getHttpMethod().name(), operationBuilder.getPath()));
  }

  public OperationBuilder addOperationBuilder(String apiOperationId, String path, String method) {
    final OperationBuilder operationBuilder = new OperationBuilder(apiOperationId, path, method, null);
    operationBuilders.add(operationBuilder);
    return operationBuilder;
  }

  public OperationBuilder getOrCreateOperationBuilder(String apiOperationId, String apiPath, String apiMethod,
                                                      String apiSummary) {
    requireNonNull(apiMethod, "API Method is required");
    requireNonNull(apiPath, "API Path is required");

    OperationBuilder operationBuilder =
        operationBuilders.stream()
            .filter(x -> x.getPath().equalsIgnoreCase(apiPath) && x.getMethod().equalsIgnoreCase(apiMethod))
            .findFirst().orElse(null);

    if (operationBuilder == null) {
      operationBuilder = new OperationBuilder(apiOperationId, apiPath, apiMethod, apiSummary);
      operationBuilders.add(operationBuilder);
    }

    return operationBuilder;
  }

  public PaginationBuilder getOrCreatePaginationBuilder(String paginationName) {
    requireNonNull(paginationName, "Pagination Name is required");

    PaginationBuilder paginationBuilder =
        paginationBuilders.stream()
            .filter(x -> x.getName().equals(paginationName))
            .findFirst().orElse(null);

    if (paginationBuilder == null) {
      paginationBuilder = new PaginationBuilder(paginationName);
      paginationBuilders.add(paginationBuilder);
    }

    return paginationBuilder;
  }

  public SecuritySchemeBuilder getOrCreateSecuritySchemeBuilder(String name,
                                                                ConnectorSecurityScheme.SecuritySchemeType type) {
    requireNonNull(name, "Security scheme name is required");

    SecuritySchemeBuilder securitySchemeBuilder =
        securitySchemeBuilders.stream()
            .filter(x -> x.getName().equals(name))
            .findFirst().orElse(null);

    if (securitySchemeBuilder == null) {
      requireNonNull(type, format("Security Scheme named %s does not exist and a type was not specified. This is a bug.", name));
      securitySchemeBuilder = new SecuritySchemeBuilder(name, type);
      securitySchemeBuilders.add(securitySchemeBuilder);
    }

    if (type != null && !securitySchemeBuilder.getType().equals(type)) {
      throw new IllegalStateException("Security scheme type does not match. This is a bug.");
    }

    return securitySchemeBuilder;
  }

  public List<TriggerBuilder> getTriggerBuilders() {
    return triggerBuilders;
  }

  public TriggerBuilder getOrCreateTriggerBuilder(String name) {
    requireNonNull(name, "Trigger name is required");

    TriggerBuilder triggerBuilder =
        triggerBuilders.stream()
            .filter(x -> x.getName().equals(name))
            .findFirst().orElse(null);

    if (triggerBuilder == null) {
      triggerBuilder = new TriggerBuilder(name);
      triggerBuilders.add(triggerBuilder);
    }

    return triggerBuilder;
  }

  public void setAllOperationsIgnored(Boolean ignoreOperations) {
    operationBuilders.forEach(x -> x.ignored(ignoreOperations));
    triggerBuilders.forEach(x -> x.ignored(ignoreOperations));
  }

  public ConnectorModel build() throws ModelGenerationException {
    requireNonNull(connectorName, "Connector name is required");

    TypeSchemaPool typeSchemaPool = new TypeSchemaPool();

    List<Pagination> paginations = paginationBuilders.stream().map(PaginationBuilder::build).collect(toList());

    OperationDisplayNameExpressionHandler operationDisplayNameExpressionHandler = buildOperationDisplayNameHandler();
    OperationIdentifierExpressionHandler operationIdentifierExpressionHandler = buildOperationIdentifierHandler();

    List<ConnectorSecurityScheme> securitySchemes = buildSecuritySchemes(typeSchemaPool);

    List<ConnectorOperation> operations = operationBuilders.stream()
        .map(x -> x.build(operationIdentifierExpressionHandler,
                          operationDisplayNameExpressionHandler,
                          typeSchemaPool,
                          paginations,
                          securitySchemes,
                          defaultInputMediaType,
                          defaultOutputMediaType,
                          buildQueryParamArrayFormat(),
                          skipOutputTypeValidation))
        .collect(toList());

    return new ConnectorModel(buildConnectorName(),
                              connectorDescription,
                              buildMavenGroup(),
                              buildMavenArtifactId(),
                              buildMavenVersion(),
                              buildBasePackage(),
                              connectorCategory,
                              baseUriBuilder.build(supportedProtocols),
                              getSupportedProtocols(),
                              operations.stream().filter(x -> !x.isIgnored()).collect(toList()),
                              paginations,
                              buildExtensionXml(),
                              triggerBuilders.stream().filter(x -> !x.isIgnored())
                                  .map(x -> x.build(operations, typeSchemaPool)).collect(toList()));
  }

  private List<ConnectorSecurityScheme> buildSecuritySchemes(TypeSchemaPool typeSchemaPool) {
    securitySchemeBuilders.add(new SecuritySchemeBuilder(UNSECURED.toString(), UNSECURED));

    return securitySchemeBuilders.stream()
        .filter(x -> x.isIgnored() == null || !x.isIgnored())
        .map(x -> x.build(typeSchemaPool, testConnectionConfig))
        .collect(toList());
  }

  private String buildConnectorName() {
    String builtName = connectorName.trim();

    if (builtName.toLowerCase().endsWith("api")) {
      builtName = builtName.substring(0, builtName.lastIndexOf("api"));
    }
    if (builtName.toLowerCase().endsWith("connector")) {
      return builtName;
    }
    return builtName + " Connector";
  }

  private String buildMavenGroup() {
    return removeMavenGroupUnwantedCharacters(mavenGroup);
  }

  private String buildMavenArtifactId() {
    return removeMavenArtifactUnwantedCharacters(mavenArtifactId);
  }

  private String buildMavenVersion() {
    return removeMavenVersionUnwantedCharacters(mavenVersion);
  }

  private List<Protocol> getSupportedProtocols() {
    return (supportedProtocols != null && !supportedProtocols.isEmpty()) ? supportedProtocols : asList(DEFAULT_PROTOCOLS);
  }

  private String buildBasePackage() {
    return isNotBlank(baseJavaPackage) ? baseJavaPackage : ConnectorPackage.buildBasePackage(connectorName);
  }

  private String buildExtensionXml() {
    return isNotBlank(extensionXml) ? extensionXml : getXmlName(connectorName);
  }

  private QueryParamArrayFormat buildQueryParamArrayFormat() {
    return defaultIfNull(queryParamArrayFormat, MULTIMAP);
  }

  private OperationIdentifierExpressionHandler buildOperationIdentifierHandler() {
    try {
      return isNotBlank(operationIdentifierExpression)
          ? new OperationIdentifierExpressionHandler(operationIdentifierExpression)
          : new OperationIdentifierExpressionHandler();

    } catch (CompilationException e) {
      throw new IllegalArgumentException("The script for the 'operationIdentifier' failed to compile. This is a bug.", e);
    }
  }

  private OperationDisplayNameExpressionHandler buildOperationDisplayNameHandler() {
    try {
      return isNotBlank(operationDisplayNameExpression)
          ? new OperationDisplayNameExpressionHandler(operationDisplayNameExpression)
          : new OperationDisplayNameExpressionHandler();
    } catch (CompilationException e) {
      throw new IllegalArgumentException("The script for the 'operationDisplayName' failed to compile. This is a bug.", e);
    }
  }
}
